
{*******************************************************}
{                                                       }
{       Demo: Collapsible sections                      }
{                                                       }
{       Copyright (c) Sergey Tkachenko                  }
{       svt@trichview.com                               }
{       http://www.trichview.com                        }
{                                                       }
{*******************************************************}

{
  Version 1.2. Required TRichView version: 17.3+
}

unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, ImgList, ComCtrls, RVStyle, RVScroll, RichView, RVTypes,
  CRVFData;

type
  TForm1 = class(TForm)
    PageControl1: TPageControl;
    TabSheet1: TTabSheet;
    TabSheet2: TTabSheet;
    ImageList1: TImageList;
    rvDemo: TRichView;
    rvFull: TRichView;
    RVStyle1: TRVStyle;
    procedure FormCreate(Sender: TObject);
    procedure rvDemoJump(Sender: TObject; id: Integer);
    function IsExpanded(rv: TCustomRichView; ItemNo: Integer): Boolean;
    procedure ChangeExpanded(rv: TCustomRichView; ItemNo: Integer);    
  private
    { Private declarations }
    procedure CreateDoc;
    procedure CopyDoc;
  public
    { Public declarations }

  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

{ TForm1 }

{ Creating a document.
  Important: "hotspots" starts paragraphs. They are followed by hypertext text
  items. All "hotspots" have unique text ("id1", "id2", ...).
  Hotspot's image index = 1 means "expanded", = 0 means "collapsed".
}
procedure TForm1.CreateDoc;
begin
   rvFull.AddHotspot('id1', 1, 1, ImageList1, 1);
   rvFull.AddNL('Topic 1', 1, -1);
   rvFull.AddNL('Bla-bla-bla. Bla-bla-bla. Bla-bla-bla. Bla-bla-bla.', 0, 0);
   rvFull.AddNL('Bla-bla-bla. Bla-bla-bla. Bla-bla-bla. Bla-bla-bla.', 0, 0);

   rvFull.AddHotspot('id2', 1, 1, ImageList1, 1);
   rvFull.AddNL('Topic 2', 1, -1);
   rvFull.AddNL('Bla-bla-bla. Bla-bla-bla. Bla-bla-bla. Bla-bla-bla.', 0, 0);
   rvFull.AddNL('Bla-bla-bla. Bla-bla-bla. Bla-bla-bla. Bla-bla-bla.', 0, 0);

   rvFull.AddHotspot('id3', 1, 1, ImageList1, 1);
   rvFull.AddNL('Topic 3', 1, -1);
   rvFull.AddNL('Bla-bla-bla. Bla-bla-bla. Bla-bla-bla. Bla-bla-bla.', 0, 0);
   rvFull.AddNL('Bla-bla-bla. Bla-bla-bla. Bla-bla-bla. Bla-bla-bla.', 0, 0);

   rvFull.AddHotspot('id4', 1, 1, ImageList1, 1);
   rvFull.AddNL('Topic 4', 1, -1);
   rvFull.AddNL('Bla-bla-bla. Bla-bla-bla. Bla-bla-bla. Bla-bla-bla.', 0, 0);
   rvFull.AddNL('Bla-bla-bla. Bla-bla-bla. Bla-bla-bla. Bla-bla-bla.', 0, 0);
   rvFull.Format;
end;
{------------------------------------------------------------------------------}
{ Copying document from rvFull to rvDemo.
  Then removing collapsed sections.
}
procedure TForm1.CopyDoc;
var i: Integer;
    NextHeadingItemNo, NextParaItemNo: Integer;
begin
  // 1. Copying. AppendFrom can copy all items except for tables and controls.
  rvDemo.Clear;
  rvDemo.AppendFrom(rvFull);
  // 2. Removing collapsed sections
  // Iterating from the last item to the first item. Storing indices of the
  // first paragraph item (in NextParaItemNo). Storing indices of the first
  // item in the heading paragraphs (in NextHeadingItemNo).
  // If the section is collapsed, removing items [NextParaItemNo..NextHeadingItemNo-1]
  NextHeadingItemNo := rvDemo.ItemCount;
  NextParaItemNo    := rvDemo.ItemCount;
  for i := rvDemo.ItemCount-1 downto 0 do
    if rvDemo.IsParaStart(i) then begin
      if (rvDemo.GetItemStyle(i) = rvsHotspot) then begin
        if not IsExpanded(rvDemo, i) then
          rvDemo.DeleteItems(NextParaItemNo, NextHeadingItemNo-NextParaItemNo);
        NextHeadingItemNo := i;
        end;
      NextParaItemNo := i;
    end;
  rvDemo.Format;
end;
{------------------------------------------------------------------------------}
procedure TForm1.FormCreate(Sender: TObject);
begin
  CreateDoc;
  CopyDoc;
end;
{------------------------------------------------------------------------------}
{ Clicking hyperlink }
procedure TForm1.rvDemoJump(Sender: TObject; id: Integer);
var
  ItemNo, i: Integer;
  RVData: TCustomRVFormattedData;
  s: TRVUnicodeString;
begin
  // GetJumpPointItemNo can be used if you do not use tables (i.e. there are
  // no hyperlinks in table cells
  rvDemo.GetJumpPointLocation(id, RVData, ItemNo);
  if RVData.GetItemStyle(ItemNo)<>rvsHotspot then
    dec(ItemNo); // Clicked on text. We assume that hypertext is just after hotspots
  // Searching the clicked hotspot in rvFull and changing its image. Then copying
  s := RVData.GetItemTextW(ItemNo);
  for i := 0 to rvFull.ItemCount-1 do
    if (rvFull.GetItemStyle(i)=rvsHotspot) and (rvFull.GetItemText(i)=s) then
    begin
      ChangeExpanded(rvFull, i);
      CopyDoc;
      exit;
    end;
end;
{------------------------------------------------------------------------------}
{ Checking the ItemNo-th item in rv. This item must be a "hotspot".
  If ImageIndex=0, it is "collapsed"
  If ImageIndex=1, it is "expanded"
}
function TForm1.IsExpanded(rv: TCustomRichView; ItemNo: Integer): Boolean;
var ImageIndex1,ImageIndex2: Integer;
    Tag: TRVTag;
    Text: TRVUnicodeString;
    il: TCustomImageList;
begin
  rv.GetHotspotInfo(ItemNo, Text, ImageIndex1, ImageIndex2, il, Tag);
  Result := ImageIndex1<>0;
end;
{------------------------------------------------------------------------------}
{ Changing image index the ItemNo-th item in rv (This item must be a "hotspot").
  1 <-> 0
}
procedure TForm1.ChangeExpanded(rv: TCustomRichView; ItemNo: Integer);
var ImageIndex1,ImageIndex2: Integer;
    Tag: TRVTag;
    Text: TRVUnicodeString;
    il: TCustomImageList;
begin
  rv.GetHotspotInfo(ItemNo, Text, ImageIndex1, ImageIndex2, il, Tag);
  if ImageIndex1=0 then
    ImageIndex1 := 1
  else
    ImageIndex1 := 0;
  rv.SetHotspotInfo(ItemNo, Text, ImageIndex1, ImageIndex1, il, Tag);
end;
{------------------------------------------------------------------------------}

end.
